import abc
import glob
import os
import unittest

from integration.partial.integration_test_utils import IntegrationTestUtils


class BaseEnvSuitTestSuite:

    class EnvSuitTestSuite(unittest.TestCase):

        """
        the name passed to the Processor.py to define the component
        """
        ENV_COMPONENT_NAME = "Environment"
        """
        the prefix of the directory created by thr Processor.py, e.g. DEPOSITION_2020-01-01
        """
        ENV_PROCESSOR_DIR = "ENVIRONMENT_2.0"

        @classmethod
        def setUpClass(cls) -> None:
            IntegrationTestUtils.TEST_OUT_PATH = None  # reset this value for each new test class


        @abc.abstractmethod
        def set_expected_values(self):
            """
            these fields need to be overridden in the tests that use this test suite
            """

            self.EA_CSV_COUNT: int = 0
            self.ETH_CSV_COUNT: int = 0
            self.EA_PNG_COUNT: int = 0
            self.ETH_PNG_COUNT: int = 0


        def test_standard_run_input_status_success(self):
            status_file_path = os.path.join(IntegrationTestUtils.TEST_OUT_PATH,
                                            IntegrationTestUtils.TEST_JOB_DIR, "STATUS_SUCCESS")
            success_file_exists: bool = os.path.isfile(status_file_path)
            self.assertTrue(success_file_exists)


        def test_standard_run_input_all_regions_ran(self):
            """
            working on the assumption that if there are images for each region, it must have run through
            (at least past the region iteration)
            """

            east_africa_image_path = os.path.join(IntegrationTestUtils.TEST_OUT_PATH,
                                                  IntegrationTestUtils.TEST_JOB_DIR,
                                                  "plotting", "eastafrica", "images", "Weekly",
                                                  "suitability_eastafrica_leaf_rust_total_*_map.png")
            ethiopia_image_path = os.path.join(IntegrationTestUtils.TEST_OUT_PATH,
                                               IntegrationTestUtils.TEST_JOB_DIR,
                                               "plotting", "ethiopia", "images", "Weekly",
                                               "suitability_ethiopia_leaf_rust_total_*_map.png")

            ea_file_exists: bool = len(glob.glob(east_africa_image_path)) > 0
            eth_file_exists: bool = len(glob.glob(ethiopia_image_path)) > 0
            self.assertTrue(ea_file_exists)
            self.assertTrue(eth_file_exists)


        def test_standard_run_all_input_csvs_produced(self):
            east_africa_csv_path = os.path.join(IntegrationTestUtils.TEST_OUT_PATH,
                                                IntegrationTestUtils.TEST_JOB_DIR,
                                                "plotting", "eastafrica", "input_csvs", "*.csv")
            ethiopia_csv_path = os.path.join(IntegrationTestUtils.TEST_OUT_PATH,
                                             IntegrationTestUtils.TEST_JOB_DIR,
                                             "plotting", "ethiopia", "input_csvs", "*.csv")

            ea_csv_count: int = IntegrationTestUtils.count_files_in_wildcard(east_africa_csv_path)
            eth_csv_count: int = IntegrationTestUtils.count_files_in_wildcard(ethiopia_csv_path)
            self.assertEqual(self.EA_CSV_COUNT, ea_csv_count)
            self.assertEqual(self.ETH_CSV_COUNT, eth_csv_count)


        def test_standard_run_all_images_produced(self):
            east_africa_image_path = os.path.join(IntegrationTestUtils.TEST_OUT_PATH,
                                                  IntegrationTestUtils.TEST_JOB_DIR,
                                                  "plotting", "eastafrica", "images", "Weekly", "*.png")
            ethiopia_image_path = os.path.join(IntegrationTestUtils.TEST_OUT_PATH,
                                               IntegrationTestUtils.TEST_JOB_DIR,
                                               "plotting", "ethiopia", "images", "Weekly", "*.png")

            ea_csv_count: int = IntegrationTestUtils.count_files_in_wildcard(east_africa_image_path)
            eth_csv_count: int = IntegrationTestUtils.count_files_in_wildcard(ethiopia_image_path)
            self.assertEqual(self.EA_PNG_COUNT, ea_csv_count)
            self.assertEqual(self.ETH_PNG_COUNT, eth_csv_count)